@@ -108,40 +108,43 @@ extern "C" {
 
 
 /* Behavior modification flags */
-extern uint_fast32_t flags;
+extern uint_fast32_t flags, a_flags;
 #define F_RECURSE		(1U << 0)
 #define F_HIDEPROGRESS		(1U << 1)
 #define F_SOFTABORT		(1U << 2)
 #define F_FOLLOWLINKS		(1U << 3)
-#define F_DELETEFILES		(1U << 4)
-#define F_INCLUDEEMPTY		(1U << 5)
-#define F_CONSIDERHARDLINKS	(1U << 6)
-#define F_SHOWSIZE		(1U << 7)
-#define F_OMITFIRST		(1U << 8)
-#define F_RECURSEAFTER		(1U << 9)
-#define F_NOPROMPT		(1U << 10)
-#define F_SUMMARIZEMATCHES	(1U << 11)
-#define F_EXCLUDEHIDDEN		(1U << 12)
-#define F_PERMISSIONS		(1U << 13)
-#define F_HARDLINKFILES		(1U << 14)
-#define F_EXCLUDESIZE		(1U << 15)
-#define F_QUICKCOMPARE		(1U << 16)
-#define F_USEPARAMORDER		(1U << 17)
-#define F_DEDUPEFILES		(1U << 18)
-#define F_REVERSESORT		(1U << 19)
-#define F_ISOLATE		(1U << 20)
-#define F_MAKESYMLINKS		(1U << 21)
-#define F_PRINTMATCHES		(1U << 22)
-#define F_ONEFS			(1U << 23)
-#define F_PRINTNULL		(1U << 24)
-#define F_PARTIALONLY		(1U << 25)
-#define F_NOCHANGECHECK		(1U << 26)
-#define F_PRINTJSON		(1U << 27)
-#define F_SKIPHASH		(1U << 28)
-#define F_PRINTUNIQUE		(1U << 29)
+#define F_INCLUDEEMPTY		(1U << 4)
+#define F_CONSIDERHARDLINKS	(1U << 5)
+#define F_RECURSEAFTER		(1U << 6)
+#define F_NOPROMPT		(1U << 7)
+#define F_EXCLUDEHIDDEN		(1U << 8)
+#define F_PERMISSIONS		(1U << 9)
+#define F_EXCLUDESIZE		(1U << 10)
+#define F_QUICKCOMPARE		(1U << 11)
+#define F_USEPARAMORDER		(1U << 12)
+#define F_REVERSESORT		(1U << 13)
+#define F_ISOLATE		(1U << 14)
+#define F_ONEFS			(1U << 15)
+#define F_PARTIALONLY		(1U << 16)
+#define F_NOCHANGECHECK		(1U << 17)
+#define F_NOTRAVCHECK		(1U << 18)
+#define F_SKIPHASH		(1U << 19)
 #define F_LOUD			(1U << 30)
 #define F_DEBUG			(1U << 31)
 
+/* Action-related flags */
+#define FA_PRINTMATCHES		(1U << 0)
+#define FA_PRINTUNIQUE		(1U << 1)
+#define FA_OMITFIRST		(1U << 2)
+#define FA_SUMMARIZEMATCHES	(1U << 3)
+#define FA_DELETEFILES		(1U << 4)
+#define FA_SHOWSIZE		(1U << 5)
+#define FA_HARDLINKFILES	(1U << 6)
+#define FA_DEDUPEFILES		(1U << 7)
+#define FA_MAKESYMLINKS		(1U << 8)
+#define FA_PRINTNULL		(1U << 9)
+#define FA_PRINTJSON		(1U << 10)
+
 /* Per-file true/false flags */
 #define FF_VALID_STAT		(1U << 0)
 #define FF_HASH_PARTIAL		(1U << 1)
@@ -269,7 +272,6 @@ extern char tempname[PATHBUF_SIZE * 2];
 
 extern const char *extensions[];
 
-extern void oom(const char * const restrict msg);
 extern void nullptr(const char * restrict func);
 extern int file_has_changed(file_t * const restrict file);
 extern int getfilestats(file_t * const restrict file);
