@@ -1,6 +1,6 @@
 /* Jody Bruchon's sorting code library
  *
- * Copyright (C) 2014-2018 by Jody Bruchon <jody@jodybruchon.com>
+ * Copyright (C) 2014-2020 by Jody Bruchon <jody@jodybruchon.com>
  * Released under The MIT License
  */
 
@@ -8,6 +8,7 @@
 #include "jody_sort.h"
 
 #define IS_NUM(a) (((a >= '0') && (a <= '9')) ? 1 : 0)
+#define IS_LOWER(a) (((a >= 'a') && (a <= 'z')) ? 1 : 0)
 
 extern int numeric_sort(const char * restrict c1,
                 const char * restrict c2, int sort_direction)
@@ -74,15 +75,22 @@ extern int numeric_sort(const char * restrict c1,
     /* Put symbols and spaces after everything else */
     } else if (*c2 < '.' && *c1 >= '.') return -sort_direction;
     else if (*c1 < '.' && *c2 >= '.') return sort_direction;
-    /* Normal strcmp() style compare */
-    else if (*c1 > *c2) return sort_direction;
-    else return -sort_direction;
+    /* Normal strcasecmp() style compare */
+    else {
+      char s1 = *c1, s2 = *c2;
+      /* Convert lowercase into uppercase */
+      if (IS_LOWER(s1)) s1 = (char)(s1 - 32);
+      if (IS_LOWER(s2)) s2 = (char)(s2 - 32);
+      if (s1 > s2) return sort_direction;
+      else return -sort_direction;
+    }
   }
 
   /* Longer strings generally sort later */
   if (len1 < len2) return -sort_direction;
   if (len1 > len2) return sort_direction;
-  /* Normal strcmp() style comparison */
+
+  /* Normal comparison - FIXME? length check should already handle these */
   if (*c1 == '\0' && *c2 != '\0') return -sort_direction;
   if (*c1 != '\0' && *c2 == '\0') return sort_direction;
 
